/*
  $Id: FIFO.ino,v 1.1 2016/08/31 08:45:12 ewan Exp $
  Store and retrieve many ordered values in the log structured memory.
  FIFO.ino (C) 2016 Ewan Parker.

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.

  The author may be reached at http//www.ewan.cc/ for queries.
*/

// Include the library to allow use of the memory as a log structure.
#include <LogStructMem.h>

// Choose and initialise the memory-specific implementation (the Arduino's
// internal EEPROM in this case).  Choose a different implementation class here
// to utilise a different type of memory (e.g. flash, DRAM, SD card).
#include <MemImplATMEGA328PEEPROM.h>
MemImplATMEGA328PEEPROM memoryType;

// Include any other class(es) to prevent Arduino specific linker errors.
#include <MemImpl.h>
#include <Wire.h>
#include <SPI.h>

// The size of the test data we are storing and retrieving.
#define TESTDATASIZE 8

// This is the structure of variables that we will save and retrieve.  By using
// a 'struct' we can store and retrieve multiple variables.
struct logmem_entry
{
  char test[TESTDATASIZE + 1];
} readData, writeData;

// Configure the log structured memory.  Provide the type of memory to use and
// the size of our data structure we intend to read and write.
LogStructMem logmemHead(&memoryType, sizeof(struct logmem_entry));
LogStructMem logmemTail(&memoryType, sizeof(struct logmem_entry));
boolean headFound, tailFound;

// Set up the serial display and retrieve the latest entry from the log
// structured memory.
void setup()
{
  Serial.begin(9600);
  Serial.println();
  Serial.println("Arduino Log Structured Memory Library.");
  Serial.println("(C) 2016 Ewan Parker.  http://www.ewan.cc/");
  Serial.println("Test storage and retrieval of memory entries in a FIFO.");
  Serial.println();

  // Read the data from the start of the log structured memory and position at
  // the end for writing data.
  headFound = logmemHead.readLatest(&writeData);
  tailFound = logmemTail.readEarliest(&readData);
}

// Loop around, adding new data to the end of the log structured memory.
void loop()
{
  // Display the data we have retrieved.
  Serial.print("head?"); Serial.print(headFound);
  if (headFound)
    { Serial.print(" wrData.test="); Serial.print(writeData.test); }
  Serial.println();
  Serial.print("tail?"); Serial.print(tailFound);
  if (tailFound)
    { Serial.print(" rdData.test="); Serial.print(readData.test); }
  Serial.println();

  // Write at the head or read from the tail?
  char c;
  Serial.print("[R]ead head or [W]rite tail> ");
  while (!Serial.available()) {}; c = Serial.read();
  if (c >= 'a' && c <= 'z') c += 'A' - 'a'; // Capitalise.
  Serial.println(c);

  switch (c)
  {
    case 'R' : // Read data from tail.
    tailFound = logmemTail.readNext(&readData);
    break;

    case 'W' : // Write data to head.
    Serial.print("DATA ["); Serial.print(TESTDATASIZE);
    Serial.print(" characters]> ");
    for (byte i = 0; i < TESTDATASIZE; i++) {
      while (!Serial.available()) {}; char c = Serial.read();
      Serial.print(c);
      writeData.test[i] = c;
    }
    writeData.test[TESTDATASIZE] = '\0';
    Serial.println();
    // Store the new data at the end of the log structured memory.
    logmemHead.writeNew(&writeData);
    break;
  }
}
