/*
  $Id: LatestConfig.ino,v 1.4 2016/08/30 12:28:51 ewan Exp $
  Store and retrieve a single up-to-date value in the log structured memory.
  LatestConfig.ino (C) 2016 Ewan Parker.

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.

  The author may be reached at http//www.ewan.cc/ for queries.
*/

// Include the library to allow use of the memory as a log structure.
#include <LogStructMem.h>

// Choose and initialise the memory-specific implementation (the Arduino's
// internal EEPROM in this case).  Choose a different implementation class here
// to utilise a different type of memory (e.g. flash, DRAM, SD card).
#include <MemImplATMEGA328PEEPROM.h>
MemImplATMEGA328PEEPROM memoryType;

// Include any other class(es) to prevent Arduino specific linker errors.
#include <MemImpl.h>

// The size of the test data we are storing and retrieving.
#define TESTDATASIZE 8

// This is the structure of variables that we will save and retrieve.  By using
// a 'struct' we can store and retrieve multiple variables.
struct logmem_entry
{
  char test[TESTDATASIZE + 1];
} cfg;

// Configure the log structured memory.  Provide the type of memory to use and
// the size of our data structure we intend to read and write.
LogStructMem logmem(&memoryType, sizeof(cfg));

// Set up the serial display and retrieve the latest entry from the log
// structured memory.
void setup()
{
  Serial.begin(9600);
  Serial.println();
  Serial.println("Arduino Log Structured Memory Library.");
  Serial.println("(C) 2016 Ewan Parker.  http://www.ewan.cc/");
  Serial.println("Test storage and retrieval of a single memory entry.");
  Serial.println();

  // Read the data from the end of the log structured memory.
  logmem.readLatest(&cfg);
}

// Loop around, adding new data to the end of the log structured memory.
void loop()
{
  // Display the data we have retrieved.
  Serial.print("test="); Serial.print(cfg.test);
  Serial.println();

  // Read some new data.
  Serial.print("DATA TO STORE ["); Serial.print(TESTDATASIZE);
  Serial.print(" characters]> ");
  for (byte i = 0; i < TESTDATASIZE; i++) {
    while (!Serial.available()) {}; char c = Serial.read();
    Serial.print(c);
    cfg.test[i] = c;
  }
  cfg.test[TESTDATASIZE] = '\0';
  Serial.println();

  // Store the new data at the end of the log structured memory.
  logmem.writeNew(&cfg);
}
