/*
  $Id: LogStructMem.h,v 1.4 2016/08/31 08:45:13 ewan Exp $
  Log structured memory Arduino library with wear levelling.
  LogStructMem.h (C) 2016 Ewan Parker.

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.

  The author may be reached at http//www.ewan.cc/ for queries.
*/

#ifndef LogStructMem_H
#define LogStructMem_H

// General definition of memory implementations.
#include <MemImpl.h>

// 2-phase EEPROM write cycles.  Enabling this protects somewhat against
// fractured writes.
//#define LOGMEM_CFG_2PHASE 1

// Debugging.
//#define LOGMEM_CFG_DEBUG 1

// This data is prepended to each block of 'user' data before writing.
typedef struct
{
  uint8_t flag; // State of the data.
  uint16_t seq; // Sequential counter.
  uint8_t size; // Total size of data.
} logmem_sys_data;

class LogStructMem
{
public:
  // Constructor.  Supply the memory type and user data length.
  LogStructMem(MemImpl *memoryType, uint8_t entrySize);

  // Scan through memory and return the earliest data.  Return 'false' if
  // nothing was found.
  boolean readEarliest(void*);

  // Return the next block of data.  Return 'false' if there is no more data.
  boolean readNext(void*);

  // Scan through memory and return the latest data.  Return 'false' if nothing
  // was found.
  boolean readLatest(void*);

  // Write a new entry (replacing the previous entry).  You will probably wish
  // to seek to the correct place in memory before calling this (e.g. by
  // calling readLatest(void*) beforehand).
  void writeNew(void*);

private:
  MemImpl *_mem;
  uint32_t _addr;
  uint8_t _len, _alen;
  logmem_sys_data _head;
  uint8_t _align_page_size(uint8_t);
};

#endif
