/*
  $Id: MemImplAT24C32.h,v 1.1 2016/08/27 07:55:03 ewan Exp $
  Atmel AT24C32 EEPROM specific memory implementation.
  MemImplAT24C32.h (C) 2016 Ewan Parker.

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.

  The author may be reached at http//www.ewan.cc/ for queries.
*/

#ifndef MemImplAT24C32_H
#define MemImplAT24C32_H

// Debugging.
//#define AT24C32_CFG_DEBUG 1

// Write verification.
// Do not use in production because this may overflow the stack.
//#define AT24C32_CFG_VERIFY 1

// The I2C address of the AT24C32.
#define AT24C32_I2C_ADDR 0x57

#include <MemImpl.h>

class MemImplAT24C32 : public MemImpl
{
public:

  // Constructor.
  MemImplAT24C32();

  // Total size of the memory.
  uint32_t getMemSize();

  // Page boundary mask for alignment of page reads and writes.
  uint8_t getBoundaryMask();

  // Read a block from memory at address mem to *data with length len.
  void readBlock(void *data, uint32_t mem, uint8_t len);

  // Write a block to memory at address mem from *data with length len.
  void writeBlock(void *data, uint32_t mem, uint8_t len);
};

#endif
