/*
  $Id: MemImplW25Q64FV.h,v 1.1 2016/08/28 16:11:44 ewan Exp $
  Winbond W25Q64FV Flash specific memory implementation.
  MemImplW25Q64FV.h (C) 2016 Ewan Parker.

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.

  The author may be reached at http//www.ewan.cc/ for queries.
*/

#ifndef MemImplW25Q64FV_H
#define MemImplW25Q64FV_H

// Debugging.
//#define W25Q64FV_CFG_DEBUG 1

// Write verification.
// Do not use in production because this may overflow the stack.
//#define W25Q64FV_CFG_VERIFY 1

#include <MemImpl.h>

class MemImplW25Q64FV : public MemImpl
{
public:
  // Constructor with the Chip Select pin.  You should wait 30us after creating
  // the new instance.
  MemImplW25Q64FV(uint8_t);

  // Total size of the memory.
  uint32_t getMemSize();

  // Page boundary mask for alignment of page reads and writes.
  uint8_t getBoundaryMask();

  // Read a block from memory at address mem to *data with length len.
  void readBlock(void *data, uint32_t mem, uint8_t len);

  // Write a block to memory at address mem from *data with length len.
  void writeBlock(void *data, uint32_t mem, uint8_t len);

private:
  uint8_t _cs;
  boolean _chipBusy(void);
  void _waitChipReady(void);
  void _chipSelect(void);
  void _chipDeSelect(void);
  void _sendSeek(uint32_t);
  void _erase4kSector(uint32_t);
};

#endif
